function [u, v] = Combinational_Weights(N, K, H)

% Inputs:
% - N:   	Sample data length;
% - K:    	Transform length;
% - H:      Coefficients of the apodization function.
%
% Outputs:
% - u:      First combinational weight;
% - v:      Second combinational weight.
%
%
% References:
% Document [1]: Gigleux, B., Vincent, F., Chaumette, E.: ‘Generalized Frequency Estimator with Rational Combination of Three Spectrum Lines', IET Radar, Sonar & Navigation, 2022
%
%
% Notes:
% Let X be a complex monochromatic signal of normalized pulsation w:
% x = exp(1j * w * (0:N-1));
%
% Let y be the signal with additive white Gaussian noise:
% y = x * exp(1j * 2*pi * rand)   +   10^(-SNR/20) * 1/sqrt(2) * (randn(size(x)) + 1j * randn(size(x)));
%
% Where SNR represents the Signal-to-Noise Ratio expressed in [dB].
%
% Let Y be the Discrete Fourier Transform of y:
% Y = fft(y.* W, K);
%
% Where H defines the apodization function, W, which can be expressed as:
% W = H(1) * cos(2*pi * 0 * (1:N)/(1+N)) + ...
%     H(2) * cos(2*pi * 1 * (1:N)/(1+N)) + ...
%     H(3) * cos(2*pi * 2 * (1:N)/(1+N)) + ...
%     H(4) * cos(2*pi * 3 * (1:N)/(1+N)) ;
%
% For example:
% - Rectangular window:     H = [+1.00000;   -0.00000;   +0.00000;   -0.00000];
% - Hanning window:         H = [+0.50000;   -0.50000;   +0.00000;   -0.00000];
% - Hamming window:         H = [  +25/46;   -0.46164;   +0.00000;   -0.00000];
% - Blackmanharris window:  H = [+0.35875;   -0.48829;   +0.14128;   -0.01168];
%
% Now, let k be the index of the maximum spectrum energy:
% [Ymax, k] = max(abs(Y).^2);
%
% Therefore the coarse normalized pulsation estimator can be expressed as:
% w_coarse  = 2*pi/K * (k - 1);
%
% Introducing the notations:
% - Early   = abs(Y(k-1))^2;
% - Prompt  = abs(Y(k  ))^2;
% - Late    = abs(Y(k+1))^2;
%
% Then, the fine normalized pulsation estimator can expressed as:
% w_fine  	= 2*pi/K * (k - 1 + alpha);
%
% With:
% alpha     = K/(2*pi) * (Early - Late) / (u * (Early + Late) + v * Prompt);


%% PROCESSING
 P_wk0_wstar                =   Energy_Spectrum( 0, N, K, H);
 P_wk1_wstar                =   Energy_Spectrum(+1, N, K, H);
dP_wk0_wstar                =  dEnergy_Spectrum( 0, N, K, H); % = 0 since dP is symmetric and wk0 is the energy peak
dP_wk1_wstar                =  dEnergy_Spectrum(+1, N, K, H);

sum_squared__P_wk__wstar	=       P_wk0_wstar^2 + ...
                               2 *  P_wk1_wstar^2 ;

sum_squared_dP_wk__wstar    =      dP_wk0_wstar^2 + ...
                               2 * dP_wk1_wstar^2 ;

% Combinational weights
u =      (sum_squared_dP_wk__wstar * P_wk1_wstar) / (sum_squared__P_wk__wstar * dP_wk1_wstar); % (cf. Document [1] - Equation (14))
v =  u * (                           P_wk0_wstar  /                              P_wk1_wstar); % (cf. Document [1] - Equation (14))