function [MSE] = Interpolation_MSE(alpha, u, v, N, K, H, SNR)

% Inputs:
% - alpha:  Fractional spectrum parameter;
% - N:   	Sample data length;
% - K:    	Transform length;
% - H:      Coefficients of the apodization function;
% - u:      First combinational weight;
% - v:      Second combinational weight;
% - SNR:    Signal-to-Noise Ratio expressed in dB.
%
% Outputs:
% - MSE:   	Assymptotic MSE of the frequency estimator.
%
%
% References:
% Document [1]: Gigleux, B., Vincent, F., Chaumette, E.: ‘Generalized Frequency Estimator with Rational Combination of Three Spectrum Lines', IET Radar, Sonar & Navigation, 2022


%% PROCESSING
% Compute the theoretical Early and Late DFT samples
[    Xe] 	= wDFT(-1 - alpha, N, K, H); % Early (cf. Document [1] - Equation (8))
[    Xp]    = wDFT( 0 - alpha, N, K, H); % Late  (cf. Document [1] - Equation (8))
[    Xl]    = wDFT(+1 - alpha, N, K, H); % Late  (cf. Document [1] - Equation (8))

% Compute theoretical Early, Prompt and Late peak values
 abs_Xe  	=  abs(Xe); % Early
 abs_Xp   	=  abs(Xp); % Prompt
 abs_Xl    	=  abs(Xl); % Late

% Notes:
% - abs_Xe can also be computed as sqrt(Energy_Spectrum(-1 - alpha, N, K, H));
% - abs_Xp can also be computed as sqrt(Energy_Spectrum( 0 - alpha, N, K, H));
% - abs_Xl can also be computed as sqrt(Energy_Spectrum(+1 - alpha, N, K, H)).

% Compute the covariance between DFT samples in the case of additive white Gaussian noise
[C0]        = DFT_Covariance(0, 0, N, K, H); % Variance of early, or prompt, or late samples (cf. Document [1] - Equation (26))
[C1]        = DFT_Covariance(0, 1, N, K, H); % Covariance between early and prompt, or prompt and late samples (cf. Document [1] - Equation (26))
[C2]        = DFT_Covariance(0, 2, N, K, H); % Covariance between early and late samples (cf. Document [1] - Equation (26))
 
% Compute the theoretical components of the estimator
U           =        abs_Xe^2 - abs_Xl^2                 ;                  % (cf. Document [1] - Equation (16))
V           =   u * (abs_Xe^2 + abs_Xl^2) + v * abs_Xp^2 ;                  % (cf. Document [1] - Equation (16))

% Compute the MSE of the numerator
MSE_dU_dU   =  2     *     ( abs_Xe^2 +  abs_Xl^2) * C0  +  ...
              -4     * real(conj(Xe)  *      Xl    * C2) ;                  % (cf. Document [1] - Equation (23))

% Compute the MSE of the numerator and the denominator
MSE_dU_dV   =  2*u   *     ( abs_Xe^2 -  abs_Xl^2) * C0  +  ...
               2*v   * real(conj(Xe)   *     Xp    * C1) +  ...
              -2*v   * real(     Xl  *  conj(Xp)   * C1) ;                  % (cf. Document [1] - Equation (24))

% Compute the MSE of the denominator
MSE_dV_dV   =  2*u^2 *     ( abs_Xe^2 +  abs_Xl^2) * C0  + ...
               2*v^2 *       abs_Xp^2              * C0  + ...
               4*u*v * real(conj(Xe)   *       Xp  * C1) + ...
               4*u*v * real(     Xl    *  conj(Xp) * C1) + ...
               4*u^2 * real(conj(Xe)   *       Xl  * C2) ;                  % (cf. Document [1] - Equation (25))

% Compute the MSE of the interpolation
MSE         =  (1/10^(SNR/10))   *   ( 1    /V^2 * MSE_dU_dU   + ...
                                      -2*U  /V^3 * MSE_dU_dV   + ...
                                         U^2/V^4 * MSE_dV_dV   ) ;          % (cf. Document [1] - Equation (21))

% Compensate residual errors
MSE         = real(MSE);