clear all
close all
clc

addpath('Functions');

% References:
% - Document [1]: Gigleux, B., Vincent, F., Chaumette, E.: ‘Generalized Frequency Estimator with Rational Combination of Three Spectrum Lines', IET Radar, Sonar & Navigation, 2022
% - Document [2]: Quinn, B.G.: ‘Frequency Estimation Using Tapered Data’, 2006 IEEE International Conference on Acoustics Speech and Signal Processing Proceedings, Toulouse, France, 2006, IEEE


%% CONFIGURATION
S       =       10000               ; % Number of draws
N       =        1024               ; % Number of samples
K       =           2 * N           ; % Transform length
Window  =              'Rectangular'; % Apodization function chosen within {'Rectangular', 'Hamming', 'Hanning', 'Blackman-Nuttall', 'Blackmanharris'}
w       =    2*pi * 0.50            ; % Normalized pulsation
delta   = linspace(-0.50, 0.50, 128); % Normalized error
SNR     =         -10.00            ; % Signal-to-Noise Ratio expressed in [dB]
G       = 10*log10(N)               ; % Processing gain


%% INITIALIZATION
% Declaration of variables
          MSE = zeros(1, length(delta));
Estimated_MSE = zeros(1, length(delta));

% Windowing
switch (Window)	
	case {'Rectangular'     }, H = [+1.0000000;   -0.0000000;   +0.0000000;   -0.0000000];
	case {'Hamming'         }, H = [  +25/46  ;     -21/46  ;   +0.0000000;   -0.0000000];
	case {'Hanning'         }, H = [+0.5000000;   -0.5000000;   +0.0000000;   -0.0000000];
    case {'Blackman-Nuttall'}, H = [+0.3635819;   -0.4891775;   +0.1365995;   -0.0106411];
    case {'Blackmanharris'  }, H = [+0.3587500;   -0.48829  ;   +0.1412800;   -0.0116800];
end

W       = H(1) * cos(2*pi * 0 * (1:N)/(1+N)) + ...
          H(2) * cos(2*pi * 1 * (1:N)/(1+N)) + ...
          H(3) * cos(2*pi * 2 * (1:N)/(1+N)) + ...
          H(4) * cos(2*pi * 3 * (1:N)/(1+N)) ;

% Combinational weights
[u, v]  = Combinational_Weights(N, K, H);


%% CRAMER-RAO LOWER BOUND AND ASYMPTOTIC VARIANCE
% Cramer-Rao Lower Bound for the estimation of a single tone with N complex samples assuming additive white Gaussian noise
CRLB            = 6 / (N * (N^2 - 1) * 10^((SNR)/10)); % (cf. Document [1] - Equation  (2))

% Asymptotic variance for frequency estimation of a single tone from the periodogram of tapered data
c3              = H(1)^2 * ( H(1)^2/24 + H(1)*H(2)/(2*pi^2) + H(2)^2/4 * (1/12 + 1/(8*pi^2)));
c4              = H(1)   * (-H(1)  / 3 -   2 *H(2)/   pi^2)                                  ;

RMSE_tappering  = 1/(2*pi) * 32*pi * c3/c4^2 / (N^3 * 10^((SNR)/10)); % (cf. Document [2] - Theorem 1)

% Note:
% Asymptotic variance has been established for rectangular, Hanning and Hamming windows in Document [2].


%% SIMULATION
% Loop over coarse error
for d = 1:length(delta)
    
    % Reset squared errors
    SE = zeros(1, S);
    
    % Generate the signal
    x   = exp(1j * (w + 2*pi * delta(d)/K) * (0:N-1));
    
    % Monte-Carlo Simulations
    for s = 1:S
        
        % Generate the noise
        b = 1/sqrt(2) * (     randn(size(x)) + ...
                         1j * randn(size(x)));
        
        % Combine signal plus noise
        y = x * exp(1j * 2*pi * rand) + 10^(-SNR/20) * b;
        
        
        % Periodogram
        Y = fft(y.* W, K);       
        
        % Coarse estimation
        [Ymax, k0] = max(abs(Y).^2);        
        
        % Residual frequency error
        Early  	= abs(Y(k0-1))^2;
        Prompt	= abs(Y(k0  ))^2;
        Late   	= abs(Y(k0+1))^2;
        
        alpha   = K/(2*pi) * (Early - Late) / (u * (Early + Late) + v * Prompt);
        
        % Fine estimation
        f       = 2*pi/K * (k0 - 1 + alpha);
        
        
        % Statistics
        SE(s)  = (f - (w + 2*pi * delta(d)/K))^2;
        
    end
    
    % Mean squared error
    MSE(d)   = mean(SE);
    
    % Estimate the variance
    [Estimated_MSE(d)] = Interpolation_MSE(delta(d), u, v, N, K, H, SNR);
    
end


%% DISPLAY
close all

Figure  = figure();
set(gcf, 'Position', [100 100 600 400]); hold on; grid on;
plot(delta, sqrt(          MSE                      / CRLB),   'b');
plot(delta, sqrt(Estimated_MSE                      / CRLB), '--c');

if strcmp(Window, 'Rectangular') || ...
   strcmp(Window, 'Hamming'    ) || ...
   strcmp(Window, 'Hanning'    )

plot(delta, sqrt(RMSE_tappering * ones(size(delta)) / CRLB), '-.r');
    
end

xlabel('\omega - \omega_0');
ylabel('RMSE/\sigma');
legend('Monte-Carlo', 'Analytical', 'Asymptotic');
title(sprintf('RMSE (SNR = %d dB, N = %d, K = %d, Window = %s)', SNR, N, K, Window));